/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl>
 *
 */

/* This header file will identity the used Attribute and Obligations version that it complies */


#ifndef ATTR_OBL_PROFILE_H
 #define ATTR_OBL_PROFILE_H

#define ATTR_OBLIG_MAJOR_VERSION 0
#define ATTR_OBLIG_MINOR_VERSION 12
#define ATTR_OBLIG_PATCH_VERSION 0



/*
 * #define STAKEHOLDER_OSG
 */


/********** B: Enum types **********/

/* Run Job, submitting to a queue */
/* #define ACTION_QUEUE            11001*/
/* Run Job, execute directly */
/*#define ACTION_EXECUTE_NOW      11002*/
/* Access a file */
/*#define ACTION_ACCESSFILE       11003*/

/* Computing Element, front-end node of the (Grid) compute cluster */
/*#define RESOURCE_CE             11101*/
/* Run Job, execute directly */
/*#define RESOURCE_SE             11102*/
/* Access a file */
/*#define RESOURCE_WN             11103*/

/********** E: Enum types **********/




/*********** B: The Subject Attributes ***********/

#ifdef REALLY_NEED_SUBJECT_ID
/* Rejected to support this */
typedef struct
{
    char * subject_id;
}
xacml_subject_id_t;
#endif


typedef struct
{
    char * subject_condor_canonical_name_id;
}
xacml_subject_condor_canonical_name_id_t;


typedef struct
{
    char * subject_x509_id;
}
xacml_subject_x509_id_t;


typedef struct
{
    char * subject_x509_issuer;
}
xacml_subject_x509_issuer_t;


typedef struct
{
    int certificate_serial_number;
}
xacml_subject_certificate_serial_number_t;


typedef struct
{
    char * subject_vo;
}
xacml_subject_vo_t;


typedef struct
{
    char * voms_signing_subject;
}
xacml_voms_signing_subject_t;


typedef struct
{
    char * voms_signing_issuer;
}
xacml_voms_signing_issuer_t;


typedef struct
{
    char * voms_dns_port;
}
xacml_voms_dns_port_t;


typedef struct
{
    char * subject_voms_fqan;
}
xacml_subject_voms_fqan_t;


typedef struct
{
    char * subject_primary_voms_fqan;
}
xacml_subject_primary_voms_fqan_t;


typedef struct
{
    char * ca_policy_oid;
}
xacml_ca_policy_oid_t;


typedef struct
{
    int ca_serial_number;
}
xacml_ca_serial_number_t;


typedef struct
{
     char * cert_chain;
}
xacml_cert_chain_t;


typedef struct 
{
#ifdef REALLY_NEED_SUBJECT_ID
    xacml_subject_id_t                          xacml_subject_id;
#endif
    xacml_subject_condor_canonical_name_id_t    xacml_subject_condor_canonical_name_id;
    xacml_subject_x509_id_t                     xacml_subject_x509_id;
    xacml_subject_x509_issuer_t                 xacml_subject_x509_issuer;
    xacml_subject_certificate_serial_number_t   xacml_subject_certificate_serial_number;
    xacml_subject_vo_t                        * xacml_subject_vo;
    int                                         xacml_subject_vo_cnt;
    xacml_voms_signing_subject_t              * xacml_voms_signing_subject;
    int                                         xacml_voms_signing_subject_cnt;
    xacml_voms_signing_issuer_t               * xacml_voms_signing_issuer;
    int                                         xacml_voms_signing_issuer_cnt;
    xacml_voms_dns_port_t                     * xacml_voms_dns_port;
    int                                         xacml_voms_dns_port_cnt;
    xacml_subject_voms_fqan_t                 * xacml_subject_voms_fqan;
    int                                         xacml_subject_voms_fqan_cnt;
    xacml_subject_primary_voms_fqan_t           xacml_subject_primary_voms_fqan;
    xacml_ca_policy_oid_t                     * xacml_ca_policy_oid;
    int                                         xacml_ca_policy_oid_cnt;
    xacml_ca_serial_number_t                  * xacml_ca_serial_number;
    int                                         xacml_ca_serial_number_cnt;
    xacml_cert_chain_t                          xacml_cert_chain;
}
xacml_profile_subject_t;


/*********** E: The Subject Attributes ***********/



/*********** B: The Action Attributes ***********/
typedef enum
{
    /* Run Job, submitting to a queue */
    ACTION_QUEUE = 11001,
    /* Run Job, execute directly */
    ACTION_EXECUTE_NOW = 11002,
    /* Access a file */
    ACTION_ACCESSFILE = 11003
}
xacml_action_job_type_enum;


typedef struct
{
    xacml_action_job_type_enum  xacml_action_job_type;
}
xacml_profile_action_t;
/*********** E: The Action Attributes ***********/



/*********** B: The Resource Attributes **********/ 
typedef enum
{
    /* Computing Element, front_end node of the (Grid) compute cluster */
    RESOURCE_CE = 11101,
    /* Run Job, execute directly */
    RESOURCE_SE = 11102,
    /* Access a file */
    RESOURCE_WN = 11103
}
xacml_resource_node_type_t;


typedef struct
{
    char * resource_dns_name;
}
xacml_resource_dns_name_t;


typedef struct
{
    xacml_resource_node_type_t xacml_resource_node_type;
    xacml_resource_dns_name_t  xacml_resource_dns_name;
}
xacml_profile_resource_t;
/*********** E: The Resource Attributes **********/ 


/*********** B: The Environment Attributes **********/ 
typedef struct
{
    char * supported_obligation;
}
xacml_environment_supported_obligation_t;


/* typedef struct
{
    char * pilot_job;
}
xacml_environment_supported_pilot_job_t; */


typedef struct
{
    xacml_environment_supported_obligation_t * xacml_environment_supported_obligation;
    int                                        xacml_environment_supported_obligation_cnt;
    xacml_profile_subject_t                    xacml_profile_subject;
}
xacml_profile_environment_t;

/*********** E: The Environment Attributes **********/ 



/*********** B: The Request message elements **********/ 
typedef struct
{
    xacml_profile_subject_t                  xacml_profile_subject;
    xacml_profile_action_t                   xacml_profile_action;
    xacml_profile_resource_t                 xacml_profile_resource;
    xacml_profile_environment_t              xacml_profile_environment;
}
xacml_profile_request_t;
/*********** E: The Request message elements **********/ 



/*********** B: The NAMESPACE declarations  **********/


/* We've made the choice to use as much already standardized elements as possible. */
#define USE_STANDARDIZED_NAMESPACE


/*** Main root prefix ***/
/*
  #define XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT "http://authz-interop.org/xacml/2.0"
  #define XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT "x-urn:authz-interop:xacml:2.0"
*/

  /* Because the attributes are independent of the used XACML version, the version is stripped */
  #define XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT "http://authz-interop.org/xacml"
  #define XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT "x-urn:authz-interop:xacml"
/*** Main root prefix ***/


/* Request element prefixes */
#define XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT       XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT"/subject"
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT       XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT":subject"

#define XACML_ATTR_PROFILE_NS_URL_PREFIX_RESOURCE      XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT"/resource"
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_RESOURCE      XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT":resource"

#define XACML_ATTR_PROFILE_NS_URL_PREFIX_ACTION        XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT"/action"
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_ACTION        XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT":action"

#define XACML_ATTR_PROFILE_NS_URL_PREFIX_ENVIRONMENT   XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT"/environment"
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_ENVIRONMENT   XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT":environment"

#define XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION    XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT"/obligation"
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION    XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT":obligation"

#define XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE     XACML_ATTR_PROFILE_NS_URL_PREFIX_ROOT"/attribute"
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE     XACML_ATTR_PROFILE_NS_URN_PREFIX_ROOT":attribute"




/* Subject section attributes */
#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_X509_ID                    XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/subject-x509-id"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_X509_ID                    XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":subject-x509-id"

#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_X509_ISSUER                XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/subject-x509-issuer"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_X509_ISSUER                XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":subject-x509-issuer"

#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_CERTIFICATE_SERIAL_NUMBER  XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/certificate-serial-number"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_CERTIFICATE_SERIAL_NUMBER  XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":certificate-serial-number"

#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_CA_SERIAL_NUMBER           XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/ca-serial-number"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_CA_SERIAL_NUMBER           XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":ca-serial-number"

#define XACML_ATTR_PROFILE_NS_URL_VO                                 XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/vo"
#define XACML_ATTR_PROFILE_NS_URN_VO                                 XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":vo"

#define XACML_ATTR_PROFILE_NS_URL_VOMS_SIGNING_SUBJECT               XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/voms-signing-subject"
#define XACML_ATTR_PROFILE_NS_URN_VOMS_SIGNING_SUBJECT               XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":voms-signing-subject"

#define XACML_ATTR_PROFILE_NS_URL_VOMS_SIGNING_ISSUER                XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/voms-signing-issuer"
#define XACML_ATTR_PROFILE_NS_URN_VOMS_SIGNING_ISSUER                XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":voms-signing-issuer"

#define XACML_ATTR_PROFILE_NS_URL_VOMS_DNS_PORT                      XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/voms-dns-port"
#define XACML_ATTR_PROFILE_NS_URN_VOMS_DNS_PORT                      XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":voms-dns-port"

#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_VOMS_FQAN                  XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/voms-fqan"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_VOMS_FQAN                  XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":voms-fqan"

#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_VOMS_PRIMARY_FQAN          XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/voms-primary-fqan"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_VOMS_PRIMARY_FQAN          XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":voms-primary-fqan"

#define XACML_ATTR_PROFILE_NS_URL_CA_POLICY_OID                      XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/ca-policy-oid"
#define XACML_ATTR_PROFILE_NS_URN_CA_POLICY_OID                      XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":ca-policy-oid"

#define XACML_ATTR_PROFILE_NS_URL_CERT_CHAIN                         XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/cert-chain"
#define XACML_ATTR_PROFILE_NS_URN_CERT_CHAIN                         XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":cert-chain"

#define XACML_ATTR_PROFILE_NS_URL_SUBJECT_CONDOR_CANONICAL_NAME_ID   XACML_ATTR_PROFILE_NS_URL_PREFIX_SUBJECT"/subject-condor-canonical-name-id"
#define XACML_ATTR_PROFILE_NS_URN_SUBJECT_CONDOR_CANONICAL_NAME_ID   XACML_ATTR_PROFILE_NS_URN_PREFIX_SUBJECT":subject-condor-canonical-name-id"


/* Obligation section IDs */
#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_UIDGID                  XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/uidgid"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_UIDGID                  XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":uidgid"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_SecondaryGIDs           XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/secondary-gids"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_SecondaryGIDs           XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":secondary-gids"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_AFSToken                XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/afs-token"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_AFSToken                XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":afs-token"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_Username                XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/username"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_Username                XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":username"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_RootAndHomePaths        XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/root-and-home-paths"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_RootAndHomePaths        XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":root-and-home-paths"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_StorageAccessPriority   XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/storage-access-priority"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_StorageAccessPriority   XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":storage-access-priority"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_AccessPermissions       XACML_ATTR_PROFILE_NS_URL_PREFIX_OBLIGATION"/access-permissions"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_AccessPermissions       XACML_ATTR_PROFILE_NS_URN_PREFIX_OBLIGATION":access-permissions"


/* Attributes for the obligations */
#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_UID          XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/posix-uid"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_UID          XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":posix-uid"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_GID          XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/posix-gid"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_GID          XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":posix-gid"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_AFSTOKEN     XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/afs-token"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_AFSTOKEN     XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":afs-token"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_USERNAME     XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/username"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_USERNAME     XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":username"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_ROOTPATH     XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/rootpath"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_ROOTPATH     XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":rootpath"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_HOMEPATH     XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/homepath"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_HOMEPATH     XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":homepath"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_PRIORITY     XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/storage-priority"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_PRIORITY     XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":storage-priority"

#define XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_ACCESSPERM   XACML_ATTR_PROFILE_NS_URL_PREFIX_ATTRIBUTE"/access-permissions"
#define XACML_ATTR_PROFILE_NS_URN_OBLIGATION_ATTRIBUTES_ACCESSPERM   XACML_ATTR_PROFILE_NS_URN_PREFIX_ATTRIBUTE":access-permissions"


/* Attributes for the action */
#ifdef USE_STANDARDIZED_NAMESPACE
    #define XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_ID               "urn:oasis:names:tc:xacml:1.0:action:action-id"
    #define XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_ID               "urn:oasis:names:tc:xacml:1.0:action:action-id"
#else
    #define XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_ID               XACML_ATTR_PROFILE_NS_URL_PREFIX_ACTION"/action-id"
    #define XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_ID               XACML_ATTR_PROFILE_NS_URN_PREFIX_ACTION":action-id"
#endif /* USE_STANDARDIZED_NAMESPACE */

#define XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE                 XACML_ATTR_PROFILE_NS_URL_PREFIX_ACTION"/action-type"
#define XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE                 XACML_ATTR_PROFILE_NS_URN_PREFIX_ACTION":action-type"

#define XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE_ACCESS_FILE     XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE"/access-file"
#define XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE_ACCESS_FILE     XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE":access-file"

#define XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE_QUEUE           XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE"/queue"
#define XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE_QUEUE           XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE":queue"

#define XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE_EXEC_NOW        XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_TYPE"/execute-now"
#define XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE_EXEC_NOW        XACML_ATTR_PROFILE_NS_URN_ACTION_ACTION_TYPE":execute-now"

/*
#define XACML_ATTR_PROFILE_NS_URL_PREFIX_ENVIRONMENT 
#define XACML_ATTR_PROFILE_NS_URN_PREFIX_ENVIRONMENT 
*/

/* Attributes for the resource */
#ifdef USE_STANDARDIZED_NAMESPACE
    #define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_ID           "urn:oasis:names:tc:xacml:1.0:resource:resource-id"
    #define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_ID           "urn:oasis:names:tc:xacml:1.0:resource:resource-id"
#else
    #define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_ID           XACML_ATTR_PROFILE_NS_URL_PREFIX_RESOURCE"/resource-id"
    #define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_ID           XACML_ATTR_PROFILE_NS_URN_PREFIX_RESOURCE":resource-id"
#endif /* USE_STANDARDIZED_NAMESPACE */

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_X509_ID          XACML_ATTR_PROFILE_NS_URL_PREFIX_RESOURCE"/resource-x509-id"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_X509_ID          XACML_ATTR_PROFILE_NS_URN_PREFIX_RESOURCE":resource-x509-id"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_X509_ISSUER      XACML_ATTR_PROFILE_NS_URL_PREFIX_RESOURCE"/resource-x509-issuer"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_X509_ISSUER      XACML_ATTR_PROFILE_NS_URN_PREFIX_RESOURCE":resource-x509-issuer"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_DNS_HOST_NAME             XACML_ATTR_PROFILE_NS_URL_PREFIX_RESOURCE"/dns-host-name"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_DNS_HOST_NAME             XACML_ATTR_PROFILE_NS_URN_PREFIX_RESOURCE":dns-host-name"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE             XACML_ATTR_PROFILE_NS_URL_PREFIX_RESOURCE"/resource-type"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE             XACML_ATTR_PROFILE_NS_URN_PREFIX_RESOURCE":resource-type"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE_CE          XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE"/ce"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE_CE          XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE":ce"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE_SE          XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE"/se"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE_SE          XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE":se"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE_WN          XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE"/wn"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE_WN          XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE":wn"

#define XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE_RB          XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_TYPE"/rb"
#define XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE_RB          XACML_ATTR_PROFILE_NS_URN_RESOURCE_RESOURCE_TYPE":rb"
 
 
#define XACML_ATTR_PROFILE_NS_URL_ENVIRONMENT_SUPPORTED_OBLIGATION   XACML_ATTR_PROFILE_NS_URL_PREFIX_ENVIRONMENT"/pep-oblig-supported"
#define XACML_ATTR_PROFILE_NS_URN_ENVIRONMENT_SUPPORTED_OBLIGATION   XACML_ATTR_PROFILE_NS_URN_PREFIX_ENVIRONMENT":pep-oblig-supported"

/*********** E: The NAMESPACE declarations  **********/



/********************************* Grid WN Profile **********************************/

#ifndef XACML_GRIDWN_OBLIGATION_LOCAL_ENVIRONMENT_MAP_POSIX
    #define XACML_GRIDWN_OBLIGATION_LOCAL_ENVIRONMENT_MAP_POSIX "http://glite.org/xacml/obligation/local-environment-map/posix"
#endif
#ifndef XACML_GRIDWN_ATTRIBUTE_USER_ID
    #define XACML_GRIDWN_ATTRIBUTE_USER_ID "http://glite.org/xacml/attribute/user-id"
#endif
#ifndef XACML_GRIDWN_ATTRIBUTE_GROUP_ID_PRIMARY
    #define XACML_GRIDWN_ATTRIBUTE_GROUP_ID_PRIMARY "http://glite.org/xacml/attribute/group-id/primary"
#endif
#ifndef XACML_GRIDWN_ATTRIBUTE_GROUP_ID
    #define XACML_GRIDWN_ATTRIBUTE_GROUP_ID "http://glite.org/xacml/attribute/group-id"
#endif

/********************************* Grid WN Profile **********************************/




#endif /* ATTR_OBL_PROFILE_H */
