dnl Licensed under the Apache License, Version 2.0 (the "License");
dnl you may not use this file except in compliance with the License.
dnl You may obtain a copy of the License at
dnl 
dnl     http://www.apache.org/licenses/LICENSE-2.0
dnl 
dnl Unless required by applicable law or agreed to in writing, software
dnl distributed under the License is distributed on an "AS IS" BASIS,
dnl WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
dnl See the License for the specific language governing permissions and
dnl limitations under the License.
dnl 
dnl Authors:
dnl 2010-
dnl     Oscar Koeroo <okoeroo@nikhef.nl>
dnl     Mischa Sall\'e <msalle@nikhef.nl>
dnl     NIKHEF Amsterdam, the Netherlands
dnl
dnl Usage: AC_PEP_API
dnl  sets *_CFLAGS and *_LIBS

AC_DEFUN([AC_PEP_API],
[
    dnl Make sure autoconf (bootstrap) fails when macro is undefined
    ifdef([PKG_CHECK_MODULES],
          [],
          [m4_fatal([macro PKG_CHECK_MODULES is not defined])])

    AC_ARG_WITH([pep-api-prefix],
		[AC_HELP_STRING([--with-pep-api-prefix=DIR],
		    [Prefix where PEP-API is installed (default /usr)])],
		[ac_pep_api_prefix=$withval],
		[ac_pep_api_prefix=/usr])

    AC_ARG_WITH([pep-api-libdir],
                [AC_HELP_STRING([--with-pep-api-libdir=DIR],
                    [Directory where PEP-API libraries are installed (default either PEP_API-PREFIX/lib or PEP_API-PREFIX/lib64)])],
                [ac_pep_api_libdir=$withval],
                [])

    ifelse([$1],[],[
	AC_PEP_API_VERSION([1])
	AC_PEP_API_VERSION([2])
    ],[
	AC_PEP_API_VERSION([$1])
	have_pep_api=$have_pep_api$1
	PEP_API_CFLAGS=$PEP_API$1_CFLAGS
	PEP_API_LIBS=$PEP_API$1_LIBS
	AC_SUBST(PEP_API_CFLAGS)
	AC_SUBST(PEP_API_LIBS)
    ])
])    

AC_DEFUN([AC_PEP_API_VERSION],
[
    ifelse([$1],[1],[
	if test "x$with_pep_api_prefix" = "x" ; then
	    PKG_CHECK_MODULES([PEP_API$1], [libargus-pep < 2], 
			      have_pep_api$1=yes, have_pep_api$1=no)
	    if test "$have_pep_api$1" = "no" ; then
		PEP_API_DIRECT_CHECK([PEP_API$1], [pep-c], [pep/pep.h],
				     have_pep_api$1=yes, have_pep_api$1=no)
	    fi
	else
	    PEP_API_DIRECT_CHECK([PEP_API$1], [pep-c], [pep/pep.h],
				 have_pep_api$1=yes, have_pep_api$1=no)
	    if test "$have_pep_api$1" = "no" ; then
		PKG_CHECK_MODULES([PEP_API$1], [libargus-pep < 2],
				  have_pep_api$1=yes, have_pep_api$1=no)
	    fi
	fi
    ],[
	if test "x$with_pep_api_prefix" = "x" ; then
	    PKG_CHECK_MODULES([PEP_API$1], [libargus-pep >= $1],
			      have_pep_api$1=yes, have_pep_api$1=no)
	    if test "$have_pep_api$1" = "no" ; then
		PEP_API_DIRECT_CHECK([PEP_API$1], [argus-pep], [argus/pep.h],
				     have_pep_api$1=yes, have_pep_api$1=no)
	    fi
	else
	    PEP_API_DIRECT_CHECK([PEP_API$1], [argus-pep], [argus/pep.h],
				 have_pep_api$1=yes, have_pep_api$1=no)
	    if test "$have_pep_api$1" = "no" ; then
		PKG_CHECK_MODULES(PEP_API$1, [libargus-pep >= $1],
				  have_pep_api$1=yes, have_pep_api$1=no)
	    fi
	fi
    ])
    
    AC_SUBST(PEP_API$1_CFLAGS)
    AC_SUBST(PEP_API$1_LIBS)
])    

dnl first argument is name, second argument library name.
AC_DEFUN([PEP_API_DIRECT_CHECK],
[
    ac_save_LIBS=$LIBS
    ac_save_CFLAGS=$CFLAGS

    dnl check whether we have specified libdir otherwise specified a prefix
    if test "x$ac_pep_api_libdir" != "x" -o "x$with_pep_api_prefix" != "x" ; then
	dnl If we haven't specified/determined a libdir, do it now from prefix
        if test "x$ac_pep_api_libdir" = "x" ; then
	    dnl For 64-bit, test if lib64 exists as non-symlink
	    if test "x$host_cpu" = "xx86_64" \
	     -a -e $with_pep_api_prefix/lib64 \
	     -a ! -h $with_pep_api_prefix/lib64 ; then
		ac_pep_api_libdir="$ac_pep_api_prefix/lib64"
	    else
		ac_pep_api_libdir="$ac_pep_api_prefix/lib"
	    fi
	fi    

	if test "x$LD_LIBRARY_PATH" != "x" ; then
            ac_save_LD_LIBRARY_PATH=$LD_LIBRARY_PATH
            export LD_LIBRARY_PATH="$ac_pep_api_libdir:$LD_LIBRARY_PATH"
        else
            export LD_LIBRARY_PATH="$ac_pep_api_libdir"
        fi

        if test "x$DYLD_LIBRARY_PATH" != "x" ; then
            ac_save_DYLD_LIBRARY_PATH=$DYLD_LIBRARY_PATH
            export DYLD_LIBRARY_PATH="$ac_pep_api_libdir:$DYLD_LIBRARY_PATH"
        else
            export DYLD_LIBRARY_PATH="$ac_pep_api_libdir"
        fi

	ac_try_CFLAGS="-I$ac_pep_api_prefix/include"
	ac_try_LIBS="-L$ac_pep_api_libdir -l$2"
	AC_MSG_CHECKING([for $1 at system default and $ac_pep_api_prefix])
    else
	ac_try_CFLAGS=""
	ac_try_LIBS="-l$2"
	AC_MSG_CHECKING([for $1 at system default])
    fi

    dnl We want to add the cmdline specified CFLAGS and LIBS, but only for
    dnl checking, not to the *_CFLAGS etc.
    CFLAGS="$ac_try_CFLAGS $CFLAGS"
    LIBS="$ac_try_LIBS $LIBS"
    AC_LINK_IFELSE([AC_LANG_SOURCE( [#include <$3>
		     int main(void) {return 0;}] )],
                    [have_mod=yes], [have_mod=no])
    AC_MSG_RESULT([$have_mod])

    if test "x$have_mod" = "xyes" ; then
	[$4]
	$1_CFLAGS=$ac_try_CFLAGS
	$1_LIBS="$ac_try_LIBS"
    else
	[$5]
	$1_CFLAGS=""
	$1_LIBS=""
    fi

    CFLAGS=$ac_save_CFLAGS
    CPPFLAGS=$ac_save_CPPFLAGS
    LIBS=$ac_save_LIBS

    if test "x$ac_save_LD_LIBRARY_PATH" = "x" ; then
	unset LD_LIBRARY_PATH
    else
	LD_LIBRARY_PATH=$ac_save_LD_LIBRARY_PATH
    fi
    if test "x$ac_save_DYLD_LIBRARY_PATH" = "x" ; then
	unset DYLD_LIBRARY_PATH
    else
	DYLD_LIBRARY_PATH=$ac_save_DYLD_LIBRARY_PATH
    fi
    if test "x$ac_save_LD_LIBRARY_PATH" = "x" ; then
        unset LD_LIBRARY_PATH
    else
        LD_LIBRARY_PATH=$ac_save_LD_LIBRARY_PATH
    fi
    if test "x$ac_save_DYLD_LIBRARY_PATH" = "x" ; then
        unset DYLD_LIBRARY_PATH
    else
        DYLD_LIBRARY_PATH=$ac_save_DYLD_LIBRARY_PATH
    fi

])
