/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#ifdef DEBUG_PEPC_PEPD
    #include <assert.h>
#endif /* DEBUG_PEPC_PEPD */
#include <string.h>

#include "pep-c-interact.h"
#include "pep-c-obligation-handlers.h"
#include "pep-c-obligation-handlers_helpers.h"

#include <lcmaps/lcmaps_log.h>


static char * pep_d_endpoint_url = NULL;
static xacml_request_t *  pepc_request   = NULL;
static xacml_response_t * pepc_response  = NULL;


/*
pep_subject_t * subject         = NULL;
pep_action_t * action           = NULL;
pep_resource_t * resource       = NULL;
pep_environment_t * environment = NULL;
*/


void setPEPdurl (char * url)
{
    pep_d_endpoint_url = (char *) url;
}

char * getPEPdURL (void)
{
    return pep_d_endpoint_url;
}

#if PEP_API == 1
int pepc_initialize(int pep_c_debug, pep_obligationhandler_t **oh) {
#else
PEP *pepc_initialize(int pep_c_debug, pep_obligationhandler_t **oh) {
#endif

    int                        oh_count = 0;
    int                        i = 0;
    int                        pep_rc = 0;

#if PEP_API == 1
    /* Initialize the PEP-C library */
    lcmaps_log_debug (5, "initialize PEP...\n");
    pep_rc= pep_initialize();
    if (pep_rc != PEP_OK) {
        lcmaps_log (LOG_ERR, "Error: pep_initialize() failed: %d: %s\n", pep_rc,pep_strerror(pep_rc));
        pep_rc= pep_destroy();
        if (pep_rc != PEP_OK) {
            lcmaps_log (LOG_ERR, "Error: pep_destroy() failed: %d: %s\n", pep_rc,pep_strerror(pep_rc));
            return pep_rc;
        }
        return -1;
    }
#else
    PEP *pep_handle;

    /* Initialize the PEP-C library */
    lcmaps_log_debug (5, "initialize PEP...\n");
    pep_handle = pep_initialize();
    if (pep_handle == NULL) {
        lcmaps_log (LOG_ERR, "Error: pep_initialize() failed\n");
        pep_destroy(pep_handle);
        return NULL;
    }
#endif


    /* Enable and disable the PEP-C library's debug setting */
    if (pep_c_debug) {
#if PEP_API == 1
        pep_setoption(PEP_OPTION_LOG_STDERR, stderr);
        pep_setoption(PEP_OPTION_LOG_LEVEL,  PEP_LOGLEVEL_DEBUG);
    } else
        pep_setoption(PEP_OPTION_LOG_LEVEL,  PEP_LOGLEVEL_NONE);
#else
        pep_setoption(pep_handle, PEP_OPTION_LOG_STDERR, stderr);
        pep_setoption(pep_handle, PEP_OPTION_LOG_LEVEL,  PEP_LOGLEVEL_DEBUG);
    } else
        pep_setoption(pep_handle, PEP_OPTION_LOG_LEVEL,  PEP_LOGLEVEL_NONE);
#endif


    /* Register the obligation handlers presented by the registerObligationHandlers() function */
    lcmaps_log_debug (5, "register Obligation Handlers: collect.\n");
    if ( registerObligationHandlers (oh, &oh_count) != 0)
#if PEP_API == 1
	return -1;
#else
	return NULL;
#endif

    /* Loop the returned list, only if it makes sense */
    for (i = 0; (i < oh_count) && 
                ( (*oh)[i].id) && 
                (strlen ( (*oh)[i].id) > 0); i++)
    {
        lcmaps_log_debug (5, "register Obligation Handlers: add obligation handler[%d] \"%s\" to pep-c.\n", i, (*oh)[i].id);
#if PEP_API == 1
        pep_rc = pep_addobligationhandler( &(*oh)[i]);
        if (pep_rc != PEP_OK) 
        {
            lcmaps_log (LOG_ERR, "Error: run_test_pep: pep_initialize() failed: %d: %s\n", pep_rc, pep_strerror(pep_rc));
            pep_destroy();
	    free(oh);
            return -1;
        }
#else
        pep_rc = pep_addobligationhandler(pep_handle, &(*oh)[i]);
        if (pep_rc != PEP_OK) 
        {
            lcmaps_log (LOG_ERR, "Error: run_test_pep: pep_initialize() failed: %d: %s\n", pep_rc, pep_strerror(pep_rc));
            pep_destroy(pep_handle);
	    free(oh);
            return NULL;
        }
#endif
    }

    /* Explicitly enable the obligation handler triggering */
#if PEP_API == 1
    pep_rc= pep_setoption(PEP_OPTION_ENABLE_OBLIGATIONHANDLERS,1);
    if (pep_rc != PEP_OK) {
        lcmaps_log (LOG_ERR, "Error: pep_setoption(PEP_OPTION_ENABLE_OBLIGATIONHANDLERS,1) failed: %s", pep_strerror(pep_rc));
        pep_destroy();
	free(oh);
        return -1;
    }
    return 0;
#else
    pep_rc= pep_setoption(pep_handle, PEP_OPTION_ENABLE_OBLIGATIONHANDLERS,1);
    if (pep_rc != PEP_OK) {
        lcmaps_log (LOG_ERR, "Error: pep_setoption(pep_handle, PEP_OPTION_ENABLE_OBLIGATIONHANDLERS,1) failed: %s", pep_strerror(pep_rc));
        pep_destroy(pep_handle);
	free(oh);
        return NULL;
    }
    return pep_handle;
#endif
}

int pepc_construct_request (
#if PEP_API != 1
                            PEP *pep_handle,
#endif
                            int use_profile,
                            const char * subject_proxy,
                            const char * actiontype,
                            const char * actionid,
                            const char * resourceid,
                            const char * resourcetype,
                            const char * dnshostname,
                            const char * environ_proxy)
{
    char * logstr = "pepc_construct_request";

    lcmaps_log_debug (5, "%s: create PEP request...\n", logstr);
	pepc_request = xacml_request_create();
    if (!pepc_request)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create pepc_request object\n", logstr);
        return 1;
    }

    if (pepc_construct_request_subject (&pepc_request, 
                                        subject_proxy))
    {
	    lcmaps_log (LOG_ERR, "%s: failed to construct request subject object\n", logstr);
        return 1;
    }
    if (pepc_construct_request_action (&pepc_request, 
                                        actiontype,
                                        actionid))
    {
	    lcmaps_log (LOG_ERR, "%s: failed to construct request action object\n", logstr);
        return 1;
    }
    if (pepc_construct_request_resource (&pepc_request, 
                                          resourceid, 
                                          resourcetype, 
                                          dnshostname))
    {
	    lcmaps_log (LOG_ERR, "%s: failed to construct request resource object\n", logstr);
        return 1;
    }
    if (pepc_construct_request_environment(&pepc_request,
                                            environ_proxy))
    {
	    lcmaps_log (LOG_ERR, "%s: failed to construct request environment object\n", logstr);
        return 1;
    }

    /* Register PIP */
    if (use_profile == PROFILE_SWITCH_GRIDWN)
    {
         /* Register the libpepc supplied PIP to transform the AuthZ Interoperability profile attributes to the Grid WN Profile 1.0 attributes */
         lcmaps_log_debug (5, "%s: Registering \"authzinterop2gridwn_adapter_pip\" PIP\n", logstr);
#if PEP_API == 1
         pep_addpip(authzinterop2gridwn_adapter_pip);
#else
         pep_addpip(pep_handle, authzinterop2gridwn_adapter_pip);
#endif
    }


    return 0;
}

int pepc_construct_request_subject    (xacml_request_t ** request, 
                                       const char * subject_proxy)
{
    char * logstr = "pepc_construct_request_subject";
	xacml_subject_t * subject = NULL;
	xacml_attribute_t * proxy = NULL;

    if ((!request) || (!(*request)))
    {
        lcmaps_log (LOG_ERR, "No request object presented, failure\n");
        return 1;
    }

	lcmaps_log_debug (5, "add PEP subject(proxy)...\n");
	subject = xacml_subject_create();

    if (!subject)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create xacml_subject_t object\n", logstr);
        return 1;
    }

	proxy = xacml_attribute_create(XACML_ATTR_PROFILE_NS_URL_CERT_CHAIN);
    if (!proxy)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create xacml_attribute_t object\n", logstr);
        return 1;
    }
	xacml_attribute_addvalue(proxy, subject_proxy);
	xacml_subject_addattribute(subject,proxy);

	xacml_request_addsubject(*request,subject);

    return 0;
}

int pepc_construct_request_action     (xacml_request_t ** request,
                                       const char * actiontype,
                                       const char * actionid)
{
    char * logstr = "pepc_construct_request_action";
	xacml_action_t * action = NULL;
	xacml_attribute_t * attr_actiontype = NULL;

    if ((!request) || (!(*request)))
    {
        lcmaps_log (LOG_ERR, "No request object presented, failure\n");
        return 1;
    }

	lcmaps_log_debug (5, "set PEP action(actiontype)...\n");
	action = xacml_action_create();
    if (!action)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create xacml_action_t object\n", logstr);
        return 1;
    }

	attr_actiontype = xacml_attribute_create(XACML_ATTR_PROFILE_NS_URL_ACTION_ACTION_ID);
    if (!attr_actiontype)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create xacml_attribute_t object\n", logstr);
        return 1;
    }
    
    if (actionid)
	    xacml_attribute_addvalue(attr_actiontype, actionid);
    else
	    xacml_attribute_addvalue(attr_actiontype, actiontype);
	xacml_action_addattribute(action, attr_actiontype);

	xacml_request_setaction(*request,action);

    return 0;
}

/***************************************************
Function:      pepc_construct_request_resource
Description:   Sets up the resource section of the 
               request
***************************************************/
int pepc_construct_request_resource   (xacml_request_t ** request,
                                       const char * resourceid,
                                       const char * resourcetype,
                                       const char * dnshostname)
{
    char * logstr = "pepc_construct_request_resource";
	xacml_resource_t * resource = NULL;
    xacml_attribute_t * attr_resourceid   = NULL;
	xacml_attribute_t * attr_dnshostname  = NULL;

    if ((!request) || (!(*request)))
    {
        lcmaps_log (LOG_ERR, "No request object presented, failure\n");
        return 1;
    }

	lcmaps_log_debug (5, "add PEP resource(resourcetype, dnshostname)...\n");
	resource = xacml_resource_create();
	
    if (!resource)
    {
        lcmaps_log (LOG_ERR, "%s: Unable to create xacml_resource_t object\n", logstr);
        return 1;
    }
        
    attr_resourceid = xacml_attribute_create(XACML_ATTR_PROFILE_NS_URL_RESOURCE_RESOURCE_ID);
    if (!attr_resourceid)
    {
        lcmaps_log (LOG_ERR, "%s: failed to created xacml_attribute_t object\n", logstr);
        return 1;
    }

    /* 
     * If there is a specific resource id specified:
     * move away from the profile document and go specs compliant
     */
    if (resourceid)
	    xacml_attribute_addvalue(attr_resourceid, resourceid);
    else
	    xacml_attribute_addvalue(attr_resourceid, resourcetype);
	xacml_resource_addattribute(resource, attr_resourceid);


	attr_dnshostname = xacml_attribute_create(XACML_ATTR_PROFILE_NS_URL_RESOURCE_DNS_HOST_NAME);
    if (!attr_dnshostname)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create xacml_attribute_t object\n", logstr);
        return 1;
    }
	xacml_attribute_addvalue(attr_dnshostname, dnshostname);
	xacml_resource_addattribute(resource, attr_dnshostname);
    
	xacml_request_addresource(*request,resource);

    return 0;
}


/*********************************************************************
Function:      pepc_construct_request_environment
Description:   Sets up the environment section of the 
               request.
               The proxy certificate used to execute the pilot job framework (not the real user job)
               will be set in the environment of the request.
*********************************************************************/
int pepc_construct_request_environment (xacml_request_t ** request,
                                        const char * environ_proxy)
{
    char * logstr = "pepc_construct_request_environment";
    char * supp_obl_id = NULL;
	xacml_environment_t * environment = NULL;
	xacml_attribute_t * proxy         = NULL;
	xacml_attribute_t * sub_obl       = NULL;

    if ((!request) || (!(*request)))
    {
        lcmaps_log (LOG_ERR, "No request object presented, failure\n");
        return 1;
    }
    
	lcmaps_log_debug (5, "set PEP environment...\n");
	environment= xacml_environment_create();
    if (!environment)
    {
        lcmaps_log (LOG_ERR, "%s: failed to create xacml_environment_t object\n", logstr);
        return 1;
    }

    /* Adding pilot proxy to environment */
    if (environ_proxy)
    {
        proxy = xacml_attribute_create(XACML_ATTR_PROFILE_NS_URL_CERT_CHAIN);
        if (!proxy)
        {
            lcmaps_log (LOG_ERR, "%s: failed to create xacml_attribute_t object\n", logstr);
            return 1;
        }

        xacml_attribute_addvalue(proxy, environ_proxy);
        xacml_environment_addattribute(environment, proxy);
    }


    /* Add supported obligations */
    rewind_supported_obligation_list();
    while ((supp_obl_id = get_next_supported_obligation_id()))
    {
        sub_obl = xacml_attribute_create(XACML_ATTR_PROFILE_NS_URL_ENVIRONMENT_SUPPORTED_OBLIGATION);
        if (!sub_obl)
        {
            lcmaps_log (LOG_ERR, "%s: failed to create xacml_attribute_t object\n", logstr);
            return 1;
        }

        xacml_attribute_addvalue(sub_obl, supp_obl_id);
        xacml_environment_addattribute(environment, sub_obl);
    }
    /* Clean up supported obligations, not needed after registration */
    /* free_supported_obligations_list(); */


    /* Add environment object to request */
	xacml_request_setenvironment(*request,environment);

    return 0;
}



#if PEP_API == 1
int pepc_engage(void)
#else
int pepc_engage(PEP *pep_handle)
#endif
{
    int pep_rc = 0;

	/* send authz request and process */
/*
	lcmaps_log_debug (3, "send PEP request to: %s\n", getPEPdURL());
#if PEP_API == 1
	pep_rc= pep_setoption(PEP_OPTION_ENDPOINT_URL, getPEPdURL());
#else
	pep_rc= pep_setoption(pep_handle, PEP_OPTION_ENDPOINT_URL, getPEPdURL());
#endif
	if (pep_rc != PEP_OK) 
    {
		lcmaps_log (LOG_ERR, "Error: pep_setoption(PEP_OPTION_ENDPOINT_URL,%s) failed: %d: %s\n", getPEPdURL(), pep_rc, pep_strerror(pep_rc));
		return 1;
	}
*/

    /* Request authorization decision */
/*
	lcmaps_log_debug (3, "pep_authorize(): %s\n", getPEPdURL());
*/
#if PEP_API == 1
	pep_rc= pep_authorize(&pepc_request,&pepc_response);
#else
	pep_rc= pep_authorize(pep_handle, &pepc_request,&pepc_response);
#endif
	if (pep_rc != PEP_OK) 
    {
		lcmaps_log (LOG_ERR, "Error: pep_authorize(request,response) failed. The Argus-PEP return code is: %d with error message: \"%s\"\n", pep_rc, pep_strerror(pep_rc));
        return 1;
	}

    return 0;
}


int pepc_get_output (lcmaps_account_info_t ** ret_account)
{
    /* int pep_rc     = 0; */
#ifdef DEBUG_PEPC_PEPD
    int i = 0, j = 0, k = 0, l = 0;
    size_t attrs_l  = 0;
    size_t values_l = 0;
    int rc = -1;
#endif /* DEBUG_PEPC_PEPD */
    char * logstr = "pepc_get_output";

	lcmaps_log_debug (5, "%s: done: process PEP response...\n", logstr);

#ifdef DEBUG_PEPC_PEPD
	xacml_request_t * request2= xacml_response_getrequest(pepc_response);
	assert(request2);
	lcmaps_log_debug (3, "PEP response.request: %s\n", (request2 == NULL) ? "NULL" : "PRESENT");
	size_t subjects_l= xacml_request_subjects_length(request2);
	lcmaps_log_debug (3, "PEP response.request: %d subjects\n", (int)subjects_l);

	for (i= 0; i<subjects_l; i++) {
		xacml_subject_t * subject= xacml_request_getsubject(request2,i);
		lcmaps_log_debug (3, "PEP response.request.subject[%d].category= %s\n", i, xacml_subject_getcategory(subject));
		attrs_l= xacml_subject_attributes_length(subject);
		lcmaps_log_debug (3, "PEP response.request.subject[%d]: %d attributes\n", i, (int)attrs_l);

		for(j= 0; j<attrs_l; j++) {
			xacml_attribute_t * attr= xacml_subject_getattribute(subject,j);
			lcmaps_log_debug (3, "PEP response.request.subject[%d].attribute[%d].id= %s\n", i,j,xacml_attribute_getid(attr));
			lcmaps_log_debug (3, "PEP response.request.subject[%d].attribute[%d].datatype= %s\n", i,j,xacml_attribute_getdatatype(attr));
			lcmaps_log_debug (3, "PEP response.request.subject[%d].attribute[%d].issuer= %s\n", i,j,xacml_attribute_getissuer(attr));
			values_l = xacml_attribute_values_length(attr);
			lcmaps_log_debug (3, "PEP response.request.subject[%d].attribute[%d]: %d values\n", i,j,(int)values_l);

			for (k= 0; k<values_l; k++) {
				lcmaps_log_debug (3, "PEP response.request.subject[%d].attribute[%d].value[%d]= %s\n", i,j,k,xacml_attribute_getvalue(attr,k));
			}
		}
	}
	size_t resources_l= xacml_request_resources_length(request2);
	lcmaps_log_debug (3, "PEP response.request: %d resources\n", (int)resources_l);
	for (i= 0; i<resources_l; i++) {
		xacml_resource_t * resource= xacml_request_getresource(request2,i);
		lcmaps_log_debug (3, "PEP response.request.resource[%d].content= %s\n", i, xacml_resource_getcontent(resource));
        
		attrs_l= xacml_resource_attributes_length(resource);
		lcmaps_log_debug (3, "PEP response.request.resource[%d]: %d attributes\n", i, (int)attrs_l);

		for(j= 0; j<attrs_l; j++) {
			xacml_attribute_t * attr= xacml_resource_getattribute(resource,j);
			lcmaps_log_debug (3, "PEP response.request.resource[%d].attribute[%d].id= %s\n", i,j,xacml_attribute_getid(attr));
			lcmaps_log_debug (3, "PEP response.request.resource[%d].attribute[%d].datatype= %s\n", i,j,xacml_attribute_getdatatype(attr));
			lcmaps_log_debug (3, "PEP response.request.resource[%d].attribute[%d].issuer= %s\n", i,j,xacml_attribute_getissuer(attr));
			values_l = xacml_attribute_values_length(attr);
			lcmaps_log_debug (3, "PEP response.request.resource[%d].attribute[%d]: %d values\n", i,j,(int)values_l);

			for (k= 0; k<values_l; k++) {
				lcmaps_log_debug (3, "PEP response.request.resource[%d].attribute[%d].value[%d]= %s\n", i,j,k,xacml_attribute_getvalue(attr,k));
			}
		}
	}
	xacml_action_t * action2= xacml_request_getaction(request2);
	lcmaps_log_debug (3, "PEP response.request.action: %s\n", (action2 == NULL) ? "NULL":"PRESENT");
	attrs_l= xacml_action_attributes_length(action2);
	lcmaps_log_debug (3, "PEP response.request.action: %d attributes\n",(int)attrs_l);

	for (j= 0; j<attrs_l; j++) {
		xacml_attribute_t * attr= xacml_action_getattribute(action2,j);
		lcmaps_log_debug (3, "PEP response.request.action.attribute[%d].id= %s\n", j,xacml_attribute_getid(attr));
		lcmaps_log_debug (3, "PEP response.request.action.attribute[%d].datatype= %s\n", j,xacml_attribute_getdatatype(attr));
		lcmaps_log_debug (3, "PEP response.request.action.attribute[%d].issuer= %s\n", j,xacml_attribute_getissuer(attr));
		values_l = xacml_attribute_values_length(attr);
		lcmaps_log_debug (3, "PEP response.request.action.attribute[%d]: %d values\n", j,(int)values_l);

		for (k= 0; k<values_l; k++) {
			lcmaps_log_debug (3, "PEP response.request.action.attribute[%d].value[%d]= %s\n",j,k,xacml_attribute_getvalue(attr,k));
		}
	}
	xacml_environment_t * env2= xacml_request_getenvironment(request2);
	lcmaps_log_debug (3, "PEP response.request.environment: %s\n", (env2 == NULL) ? "NULL":"PRESENT");
	attrs_l= xacml_environment_attributes_length(env2);
	lcmaps_log_debug (3, "PEP response.request.environment: %d attributes\n",(int)attrs_l);
	for (j= 0; j<attrs_l; j++) {
		xacml_attribute_t * attr= xacml_environment_getattribute(env2,j);
		lcmaps_log_debug (3, "PEP response.request.environment.attribute[%d].id= %s\n", j,xacml_attribute_getid(attr));
		lcmaps_log_debug (3, "PEP response.request.environment.attribute[%d].datatype= %s\n", j,xacml_attribute_getdatatype(attr));
		lcmaps_log_debug (3, "PEP response.request.environment.attribute[%d].issuer= %s\n", j,xacml_attribute_getissuer(attr));
		values_l = xacml_attribute_values_length(attr);
		lcmaps_log_debug (3, "PEP response.request.environment.attribute[%d]: %d values\n", j,(int)values_l);
 
		for (k= 0; k<values_l; k++) {
			lcmaps_log_debug (3, "PEP response.request.environment.attribute[%d].value[%d]= %s\n",j,k,xacml_attribute_getvalue(attr,k));
		}
	}


	size_t results_l= xacml_response_results_length(pepc_response);
	lcmaps_log_debug (5, "PEP response: %d results\n", (int)results_l);
    
    if ((int)results_l == 0)
    {
        lcmaps_log (LOG_ERR, "Error: No results returned from PEP-C to PEP-daemon interaction\n");
        return 1;
    }
        
	for(i= 0; i<results_l; i++)
    {
        /* int oblig_handled = 0; */
        
		xacml_result_t * result= xacml_response_getresult(pepc_response,i);
		assert(result);
		lcmaps_log_debug (5, "PEP response.result[%d].decision= %d\n", i, xacml_result_getdecision(result));
		lcmaps_log_debug (5, "PEP response.result[%d].resourceid= %s\n", i, xacml_result_getresourceid(result));
        
		xacml_status_t * status= xacml_result_getstatus(result);
		lcmaps_log_debug (5, "PEP response.result[%d].status: %s\n", i, (status == NULL) ? "NULL" : "PRESENT");
		assert(status);
		lcmaps_log_debug (5, "PEP response.result[%d].status.message= %s\n", i, xacml_status_getmessage(status));
		xacml_status_code_t * statuscode= xacml_status_getcode(status);
		lcmaps_log_debug (5, "PEP response.result[%d].status.code: %s\n", i, (statuscode == NULL) ? "NULL" : "PRESENT");
		assert(statuscode);
		lcmaps_log_debug (5, "PEP response.result[%d].status.code.code= %s\n", i, xacml_status_code_getvalue(statuscode));
		xacml_status_code_t * subcode= xacml_status_code_getsubcode(statuscode);
		lcmaps_log_debug (5, "PEP response.result[%d].status.code.subcode: %s\n", i, (subcode == NULL) ? "NULL" : "PRESENT");
		size_t obligations_l= xacml_result_obligations_length(result);
		lcmaps_log_debug (5, "PEP response.result[%d]: %d obligations\n", i, (int)obligations_l);
        
        if ((int)obligations_l == 0)
        {
            lcmaps_log (LOG_ERR, "Error: No obligations returned in response message\n");
            return 2;
        }
        
		for(j= 0; j<obligations_l; j++) 
        {
			xacml_obligation_t * obligation= xacml_result_getobligation(result,j);
			assert(obligation);
			lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].id= %s\n",i,j, xacml_obligation_getid(obligation));

            if (strcmp (XACML_ATTR_PROFILE_NS_URL_OBLIGATION_UIDGID, xacml_obligation_getid(obligation)) == 0)
            {
            
                lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].fulfillOn= %d\n",i,j, xacml_obligation_getfulfillon(obligation));

                attrs_l= xacml_obligation_attributeassignments_length(obligation);
                lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d]: %d attribute assignments\n",i,j,(int)attrs_l);

                for (k= 0; k<attrs_l; k++) 
                {
                    xacml_attribute_assignment_t * attr= xacml_obligation_getattributeassignment(obligation,k);
                    assert(attr);

                    lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].attributeassignment[%d].id= %s\n",i,j,k,xacml_attribute_assignment_getid(attr));
                    values_l = xacml_attribute_assignment_values_length(attr);

                    if (strcmp (XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_UID, xacml_attribute_assignment_getid(attr)) == 0)
                    {
                        for (l= 0; l<values_l; l++) 
                        {
                            uid_t uid = -1;
                            lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].attributeassignment[%d].value[%d]= %s\n",i,j,k,l,xacml_attribute_assignment_getvalue(attr,l));
                            uid = (uid_t) atoi (xacml_attribute_assignment_getvalue(attr,l));
                            lcmaps_log (5, "%s: Adding UID :  %d\n", logstr, uid);
                            rc = addCredentialData(UID, &uid);
                        }
                    }
                    else if (strcmp (XACML_ATTR_PROFILE_NS_URL_OBLIGATION_ATTRIBUTES_GID, xacml_attribute_assignment_getid(attr)) == 0)
                    {
                        for (l= 0; l<values_l; l++) 
                        {
                            gid_t gid = -1;
                            lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].attributeassignment[%d].value[%d]= %s\n",i,j,k,l,xacml_attribute_assignment_getvalue(attr,l));
                            gid = (gid_t) atoi (xacml_attribute_assignment_getvalue(attr,l));
                            lcmaps_log (5, "%s: Adding GID :  %d\n", logstr, gid);
                            rc = addCredentialData(PRI_GID, &gid);
                        }
                    }
                }
            }


            else if (strcmp (XACML_ATTR_PROFILE_NS_URL_OBLIGATION_SecondaryGIDs, xacml_obligation_getid(obligation)) == 0)
            {

                lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].fulfillOn= %d\n",i,j, xacml_obligation_getfulfillon(obligation));

                attrs_l= xacml_obligation_attributeassignments_length(obligation);
                lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d]: %d attribute assignments\n",i,j,(int)attrs_l);

                for (k= 0; k<attrs_l; k++) 
                {
                    xacml_attribute_assignment_t * attr= xacml_obligation_getattributeassignment(obligation,k);
                    assert(attr);

                    lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].attributeassignment[%d].id= %s\n",i,j,k,xacml_attribute_assignment_getid(attr));
                    values_l = xacml_attribute_assignment_values_length(attr);

                    for (l= 0; l<values_l; l++) {
                        lcmaps_log_debug (5, "PEP response.result[%d].obligation[%d].attributeassignment[%d].value[%d]= %s\n",i,j,k,l,xacml_attribute_assignment_getvalue(attr,l));
                    }
                }

            }

            else
            {
                lcmaps_log (LOG_ERR, "Error: Can't handle obligation id: %s\n", xacml_obligation_getid(obligation));
                return 10;
            }
		}
	}


#endif /* DEBUG_PEPC_PEPD: PEP-C to PEP-D interaction */
	return 0;
}



#if PEP_API == 1
int pepc_cleanup (void)
{
    int pep_rc = 0;
#else
int pepc_cleanup (PEP *pep_handle)
{
#endif
    
	lcmaps_log_debug (5, "delete PEP request and response...\n");
	xacml_request_delete(pepc_request);
	xacml_response_delete(pepc_response);

	lcmaps_log_debug (5, "destroy PEP...\n");
#if PEP_API == 1
	pep_rc= pep_destroy();
	if (pep_rc != PEP_OK) {
		lcmaps_log (LOG_ERR, "Error: pep_destroy() failed: %d: %s\n",pep_rc,pep_strerror(pep_rc));
		return pep_rc;
	}
#else
	pep_destroy(pep_handle);
#endif

	lcmaps_log_debug (5, "done.\n");
	return 0;
}

