/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl>
 *
 */

/* Next define is necessary for OpenSolaris for getpwnam_r() and getpwuid_r() */
#ifdef sun
#define _POSIX_PTHREAD_SEMANTICS
#endif

#include <lcmaps/lcmaps_account.h>
/* For lcmaps_get_gidlist */
#include <lcmaps/lcmaps_modules.h>

#include "pep-c-obligation-handlers_helpers.h"

#include <errno.h>


static lcmaps_account_info_t * global_account = NULL;
static list_supported_obligations_t * supp_obl_list = NULL;
static list_supported_obligations_t * supp_obl_list_index = NULL;


char * get_next_supported_obligation_id (void)
{
    char * tmp = NULL;

    /* Should have an index, otherwise we're at the end of the list or no list available */
    if (supp_obl_list_index)
    {
        tmp = supp_obl_list_index -> obligation_id;
        supp_obl_list_index = supp_obl_list_index -> next;

        return tmp;
    }
    else
        return NULL;
}


void rewind_supported_obligation_list (void)
{
    supp_obl_list_index = NULL;
}


int print_supported_obligations_list (void)
{
    list_supported_obligations_t * tmp = supp_obl_list;
    while (tmp)
    {
        lcmaps_log_debug (5, "Registered supported obligations: %s\n", tmp -> obligation_id);
        tmp = tmp -> next;
    }
    return 0;
}


int add_supported_obligation_to_list (const char * obligation_id)
{
    list_supported_obligations_t * node = NULL;
    list_supported_obligations_t * tmp  = NULL;

    node = calloc (1, sizeof(list_supported_obligations_t));

    if (!node)
    {
        return -1;
    }
    else
    {
        if (!(node -> obligation_id = strdup(obligation_id)))
        {
            free (node);
            return -1;
        }
        node -> next = NULL;

        if (!supp_obl_list)
        {
            /* Add node as first in the list */
            supp_obl_list = node;
        }
        else
        {
            tmp = supp_obl_list;

            /* Loop to final struct in the list */
            while (tmp -> next)
            { 
                tmp = tmp -> next; 
            }

            /* Add node to list */
            tmp -> next = node;
        }
    }

    /* print_supported_obligations_list(); */
    return 0;
}


int free_supported_obligations_list (void)
{
    list_supported_obligations_t * tmp = NULL;
    while (supp_obl_list)
    {
        tmp = supp_obl_list;
        supp_obl_list = supp_obl_list -> next;
        free(tmp->obligation_id);
        free(tmp);
    }
    return 0;
}


lcmaps_account_info_t * getAccountInfo ()
{
    return global_account;
}


int addCredentialDataFromUsername (const char * username)
{
    int rc = -1;
    int j  = 0;
    uid_t uid = -1;
    gid_t gid = -1;
    int cnt_sec_gid = 0;
    gid_t * sec_gid = NULL;
    struct passwd * pw = NULL;

    if (! (username && (strlen(username) > 0)))
    {
        lcmaps_log (LOG_ERR, "%s: Error: Couldn't find the Username value with the attribute identifier. This is a protocol error.\n");
        return 1;
    }

    pw = getpwnam(username);
    if (!pw)
    {
        lcmaps_log (LOG_ERR,
		"%s: Error: getpwnam() did not find the username '%s'.\n",
		__func__, username);
        return 1;
    }
    else
    {
        uid = pw -> pw_uid;
        gid = pw -> pw_gid;

        lcmaps_log_debug (3, "%s: Adding UID :  %d\n", __func__, uid);
        rc  = addCredentialData(UID, &uid);
        lcmaps_log_debug (3, "%s: Adding GID :  %d\n", __func__, gid);
        rc += addCredentialData(PRI_GID, &gid);

        /*
         * Retrieve secondary group id's
         */
        if (lcmaps_get_gidlist(username, &cnt_sec_gid, &sec_gid)==0)
        {
            for (j = 0; j < cnt_sec_gid; j++)
            {
                lcmaps_log_debug (3, "%s: Adding SGID :  %d\n", __func__, sec_gid[j]);
                rc += addCredentialData(SEC_GID, &(sec_gid[j]));
            }
            free(sec_gid);
        }
    }

    return rc;
}

#if 0
/******************************************************************************
Function:   get_gidlist()

Description:
    Finds the list of gids for user in the group file (/etc/group)
    Returns a list of gid_t which should be freed by calling program.

Parameters:
    username:   the name of the user
    ngroups:    ptr to int which will be filled with the number of gids.
    group_list: ptr to an array of gid_t.

Returns:
    0 on success. 
    -1 on realloc failure
    -2 on getgrent failure
    1 on failure
******************************************************************************/
int get_gidlist(
        const char * username,
        int * ngroups,
        gid_t ** group_list
    )
{
    struct group        * group_info = NULL;
    gid_t               * groups = NULL;
    gid_t               * newgroups = NULL;
    int                   i = 0;

    /* rewind the file pointer to the beginning of the /etc/group file */
    setgrent();

    lcmaps_log_debug(2, "\tlcmaps_get_gidlist(): looping through group file\n");
    *ngroups = 0;
    while ( ( group_info = getgrent() ) )
    {
        char ** pgr_mem = group_info->gr_mem;
        char *  gr_mem = NULL;

        lcmaps_log_debug(4, "\tlcmaps_get_gidlist(): group %s\n", group_info->gr_name);
        while ( (gr_mem = *pgr_mem) )
        {
            lcmaps_log_debug(4, "\tlcmaps_get_gidlist(): \tgroup member %s\n", gr_mem);
            if (strncmp(username, gr_mem, strlen(username))==0)
            {
                lcmaps_log_debug(2, "\tlcmaps_get_gidlist(): \t\tfound group %s for %s\n",
                    group_info->gr_name, username);
                (*ngroups)++;
                newgroups = (gid_t *) realloc(groups, ((*ngroups) * sizeof(gid_t)));
                if (newgroups == NULL)
                {
                    lcmaps_log (LOG_ERR, "lcmaps_get_gidlist(): cannot realloc\n");
                    free(groups);
                    return -1;
                }
                groups=newgroups;
                groups[(*ngroups)-1] = group_info->gr_gid;
            }
            ++pgr_mem;
        }
    }
    if (errno==ENOMEM)
    {
        lcmaps_log (LOG_ERR, "lcmaps_get_gidlist(): Cannot read the group file, %s\n", strerror(errno));
        free(groups);
        groups=NULL;
        /* Close the group file */
        endgrent();
        return -2;
    }
    *group_list=groups;
    lcmaps_log_debug(4,"\tlcmaps_get_gidlist(): %d groups found for %s\n", *ngroups, username);
    for (i = 0; i < *ngroups; i++)
    {
        lcmaps_log_debug(4,"\tlcmaps_get_gidlist(): group nr %d ==> gid_t %d\n", i+1, groups[i]);
    }
    /* Close the group file */
    endgrent();
    return 0;
}
#endif
 



/**
 * Returns the string representation of the decision.
 */     
const char * decision_str(int decision)
{
    switch (decision) 
    {
        /* The following construct of pre-compiler works and is better then the bare '0' and '1' values 
         * it switched on in the example. Never the less, this should be improved */
      #if XACML_DECISION_DENY == XACML_FULFILLON_DENY
        case XACML_DECISION_DENY:
      #else
        case XACML_DECISION_DENY:
        case XACML_FULFILLON_DENY:
      #endif
            return "Deny";

      #if XACML_DECISION_PERMIT == XACML_FULFILLON_PERMIT
        case XACML_FULFILLON_PERMIT:
      #else
        case XACML_FULFILLON_PERMIT:
        case XACML_DECISION_PERMIT:
      #endif
            return "Permit";

        case XACML_DECISION_INDETERMINATE:
            return "Indeterminate";

        case XACML_DECISION_NOT_APPLICABLE:
            return "Not Applicable";

        default:
            return "Deny (Unknown!?!)";
    }
}



/*
 * This function will test if the response message contains the right 
 * amount of result sets and if the results are all OK and have a Permit 
 * decision 
 */
int checkResponseSanity (xacml_response_t * response)
{
    int i = 0;
    size_t results_l           = 0;

    lcmaps_log_debug (5, "%s\n", __func__);

    if (response == NULL)
    {
        lcmaps_log_debug (5, "%s: response is NULL\n", __func__);
        return PEP_ERR_NULL_POINTER;
    }

    results_l = xacml_response_results_length(response);
    lcmaps_log_debug (5, "%s: response: %d results\n", __func__, (int)results_l);
    
    /* No results == fail! */
    if ((int)results_l == 0)
    {
        lcmaps_log (LOG_ERR, "%s: Error: No results in response message. Communication failure between or unknown failure at the PEP Daemon\n", __func__);
        return C_PEP_FAIL;
    }

    /* Too many results == fail! */
    if ((int)results_l > 1)
    {
        lcmaps_log (LOG_ERR, "%s: Error: %d results in response message found. I can only handle one result from the PEP Daemon\n", __func__, (int)results_l);
        return C_PEP_FAIL;
    }


    /* Although I fail on anything else then one argument, for the sake of code completeness 
     * I'll keep the for-loop on the amount of results */
    for (i = 0; i < results_l; i++)
    {
        xacml_result_t * result = xacml_response_getresult(response,i);

        if (xacml_result_getdecision(result) != XACML_DECISION_PERMIT)
        {
            lcmaps_log (LOG_ERR, "%s: Error: the decision for result[%d] is %s. This means your request is not allowed to continue based on this decision.\n", 
                           __func__, i, decision_str(xacml_result_getdecision(result)));
            return C_PEP_FAIL;
        }
        else
        {
            lcmaps_log_debug (5, "%s: response.result[%d].decision= %s\n", __func__, i, decision_str(xacml_result_getdecision(result)));
        }

        /* Resource-id: unknown */
        lcmaps_log_debug (5, "%s: response.result[%d].resourceid= %s\n", __func__, i, xacml_result_getresourceid(result));
        
        /* Status */
        xacml_status_t * status = xacml_result_getstatus(result);
        lcmaps_log_debug (5, "%s: response.result[%d].status.message= %s\n", __func__, i, xacml_status_getmessage(status));

        /* Status code */
        xacml_statuscode_t * statuscode = xacml_status_getcode(status);
        if (strncmp (XACML_STATUSCODE_OK,  xacml_statuscode_getvalue(statuscode), strlen(XACML_STATUSCODE_OK)) != 0)
        {
            lcmaps_log (LOG_ERR, "%s: Error: the statuscode of result[%d] is: %s. This means the request is not allowed to continue based on this status code.\n", 
                           __func__, i, xacml_statuscode_getvalue(statuscode));
            return C_PEP_FAIL;
        }
        else
        {
            lcmaps_log_debug (5, "%s: response.result[%d].status.code.code= %s\n", __func__, i, xacml_statuscode_getvalue(statuscode));
        }

        /* Status sub code */
        xacml_statuscode_t * subcode = xacml_statuscode_getsubcode(statuscode);
        if (subcode != NULL)
        {
            if (strncmp (XACML_STATUSCODE_OK, xacml_statuscode_getvalue(subcode), strlen(XACML_STATUSCODE_OK)) != 0)
            {
                lcmaps_log (LOG_ERR, "%s: Error: the statuscode of result[%d] is: %s. This means the request is not allowed to continue based on this status code.\n", 
                               __func__, i, xacml_statuscode_getvalue(statuscode));
                return C_PEP_FAIL;
            }
            else
            {
                lcmaps_log_debug (5, "%s: response.result[%d].status.code.subcode.code= %s\n", __func__, i, xacml_statuscode_getvalue(subcode));
            }
        }
    }

    /* All is ok */
    return C_PEP_OK;
}


uid_t threadsafe_getuid_from_name (const char * username)
{
    struct passwd * p          = NULL;
    uid_t           uid        = -1;
    int             errno_save = 0;
#ifndef HAVE_GETPWNAM_R /* Temporary hack */
    #define HAVE_GETPWNAM_R
#endif
#ifdef HAVE_GETPWNAM_R
    char *          pwtmpbuf = NULL;
    struct passwd   pwbuf;
    size_t          buflen = 1024;
  #ifdef _SC_GETPW_R_SIZE_MAX
    buflen = sysconf (_SC_GETPW_R_SIZE_MAX);
    if ((buflen == -1) && (errno != 0))
    {
        buflen = 1024;
    }
  #endif

    /* Allocate the buffer for getpwnam_r() */
    pwtmpbuf = (char *) calloc (1, buflen);

    /* Get Passwd Info for Username */
    while (getpwnam_r (username, &pwbuf, pwtmpbuf, buflen, &p) != 0)
    {
        /* If the errno is anything different then ERANGE, something is wrong or the ID is not available */
        if (errno != ERANGE)
        {
            errno_save = errno;
            p = NULL;
            break;
        }

        /* ERANGE is left over, which means that there is insufficient buffer space. Increasing buffer side and try again */
        buflen *= 2;
        free (pwtmpbuf);

        /* Allocate the buffer for getpwnam_r() */
        if (!(pwtmpbuf = (char *) calloc (1, buflen)))
        {
            /* System ran out of memory */
            if (errno == ENOMEM)
            {
                /* No memory available, bail out */
                p = NULL;
                errno_save = errno;
                break;
            }
        }
    }
#else
    p = getpwnam (username);
#endif

    /* If there is an entry found, fetch the UID info */
    if (p)
    {
        uid = p -> pw_uid;
    }

    /* Remove the buffer for the getpwnam_r() */
    free (pwtmpbuf);
    errno = errno_save;
    return uid;
}


gid_t threadsafe_getgid_from_name (const char * groupname)
{
    struct group  * g          = NULL;
    gid_t           gid        = -1;
    int             errno_save = 0;
#ifndef HAVE_GETGRNAM_R /* Temporary hack */
    #define HAVE_GETGRNAM_R
#endif
#ifdef HAVE_GETGRNAM_R
    char *          grtmpbuf = NULL;
    struct group    grbuf;
    size_t          buflen = 1024;
  #ifdef _SC_GETGR_R_SIZE_MAX
    buflen = sysconf (_SC_GETGR_R_SIZE_MAX);
    if ((buflen == -1) && (errno != 0))
    {
        buflen = 1024;
    }
  #endif

    /* Allocate the buffer for getpwnam_r() */
    grtmpbuf = (char *) calloc (1, buflen);

    /* Get Passwd Info for Username */
    while (getgrnam_r (groupname, &grbuf, grtmpbuf, buflen, &g) != 0)
    {
        /* If the errno is anything different then ERANGE, something is wrong or the ID is not available */
        if (errno != ERANGE)
        {
            errno_save = errno;
            g = NULL;
            break;
        }

        /* ERANGE is left over, which means that there is insufficient buffer space. Increasing buffer side and try again */
        buflen *= 2;
        free (grtmpbuf);

        /* Allocate the buffer for getpwnam_r() */
        if (!(grtmpbuf = (char *) calloc (1, buflen)))
        {
            /* System ran out of memory */
            if (errno == ENOMEM)
            {
                /* No memory available, bail out */
                g = NULL;
                errno_save = errno;
                break;
            }
        }
    }
#else
    /* If the platform doesn't have a getgrnam_r() function */
    g = getgrnam (username);
#endif

    /* If there is an entry found, fetch the GID info */
    if (g)
    {
        gid = g -> gr_gid;
    }

    /* Remove the buffer for the getgrnam_r() */
    free (grtmpbuf);
    errno = errno_save;
    return gid;
}

